{*****************************************************************************
 *
 *  WizardUnit.pas - New Route Wizard
 *
 *  Copyright (c) 1999 Diego Amicabile
 *
 *  Author:     Diego Amicabile
 *  E-mail:     diegoami@yahoo.it
 *  Homepage:   http://www.geocities.com/diegoami
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation;
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *----------------------------------------------------------------------------
 *
 *  Revision history:
 *
 *     DATE     REV                 DESCRIPTION
 *  ----------- --- ----------------------------------------------------------
 *
 *****************************************************************************}

 unit WizardUnit;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, Grids, DBGrids, PageMngr, dialogs;

type
  TNewRouteWizard = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    NextButton: TButton;
    Notebook: TNotebook;
    EnterTownPanel: TPanel;
    AddTownSpeedButton: TSpeedButton;
    RemoveTownSpeedButton: TSpeedButton;
    ChangeTownSpeedButton: TSpeedButton;
    RoutesDBGrid: TDBGrid;
    TownNameLabel: TLabel;
    RouteNameLabel: TLabel;
    OtherWayLabel: TLabel;
    TownsEdit: TEdit;
    OtherWayEdit: TEdit;
    TownsListBox: TListBox;
    DBGrid: TDBGrid;
    PageManager: TPageManager;
    NewRoutePageProxy: TPageProxy;
    RouteToPageProxy: TPageProxy;
    RouteBackPageProxy: TPageProxy;
    BackButton: TButton;
    RouteNameEdit: TEdit;
    procedure AddTownSpeedButtonClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure RemoveTownSpeedButtonClick(Sender: TObject);
    procedure ChangeTownSpeedButtonClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure NewRoutePageProxyEnter(Next: Boolean);
    procedure FormShow(Sender: TObject);
    procedure RouteNameEditKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TownsEditKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure NewRoutePageProxyLeave(Next: Boolean);
    procedure RouteToPageProxyLeave(Next: Boolean);
    procedure FormResize(Sender: TObject);
    procedure RouteBackPageProxyLeave(Next: Boolean);
    procedure RouteNameEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure RouteNameEditKeyPress(Sender: TObject; var Key: Char);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);


  private
    procedure ClearAll;
  public
    { Public declarations }
  end;

var
  NewRouteWizard: TNewRouteWizard;

implementation

uses DataMod;

{$R *.DFM}
procedure TNewRouteWizard.ClearAll;
begin
  TownsEdit.Clear;
  RouteNameEdit.Clear;
  TownsListBox.Items.Clear;
end;

function Reverse(Ts : TStrings) : TStrings;
var Ls : TStringList;
  i  : smallint;
  aString : String;
begin
  ls := TStringList.Create;
  for i := Ts.Count-1 downto 0 do begin
    aString := ts.Strings[i];
      Ls.add(aString);
  end;
  result := Ls;
end;

procedure TNewRouteWizard.AddTownSpeedButtonClick(Sender: TObject);
begin
  if TownsListBox.ItemIndex = -1 then
    TownsListBox.Items.Add(TownsEdit.Text)
  else
    TownsListBox.Items.Insert(TownsListBox.ItemIndex+1,TownsEdit.Text);
end;

procedure TNewRouteWizard.CancelBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TNewRouteWizard.RemoveTownSpeedButtonClick(Sender: TObject);
begin
  TownsListBox.Items.Delete(TownsListBox.ItemIndex);
end;

procedure TNewRouteWizard.ChangeTownSpeedButtonClick(Sender: TObject);
begin
  if TownsListBox.ItemIndex > -1 then
    TownsListBox.Items.Strings[TownsListBox.ItemIndex] := TownsEdit.Text;
end;

procedure TNewRouteWizard.OkBtnClick(Sender: TObject);
begin
  RoutesDataModule.RoutesTable.Edit;
  RoutesDataModule.RoutesTable.Post;
  ClearAll;
  Close;
end;


procedure TNewRouteWizard.NewRoutePageProxyEnter(Next: Boolean);
begin
  NextButton.Enabled := False;
end;

procedure TNewRouteWizard.FormShow(Sender: TObject);
begin
  NextButton.Enabled := False;
end;

procedure TNewRouteWizard.RouteNameEditKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Length(RouteNameEdit.Text) > 0) and (Length(OtherWayEdit.Text) > 0)
      and (TownsListBox.Items.Count > 0 ) then
    NextButton.Enabled := True
  else
    NextButton.Enabled := False;
end;

procedure TNewRouteWizard.TownsEditKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = 13 then begin
    AddTownSpeedButtonClick(Sender);
    TownsEdit.Text := '';
  end;
  RouteNameEditKeyUp(Sender,Key,Shift);
end;

procedure TNewRouteWizard.NewRoutePageProxyLeave(Next: Boolean);
begin
  if Next then begin { with .. do sucks }
    RoutesDataModule.CreateTable(RouteNameEdit.Text,TownsListBox.Items);
    RoutesDataModule.RoutesTable.Active      := False;
    RoutesDataModule.RoutesTable.TableName     := RouteNameEdit.Text+'.db';
    RoutesDataModule.RoutesTable.Active      := True;
  end;
  OkBtn.Enabled := True;
end;

procedure TNewRouteWizard.RouteToPageProxyLeave(Next: Boolean);
begin
  if Next then begin
    RoutesDataModule.CreateTable(OtherWayEdit.Text,Reverse(TownsListBox.Items));
    RoutesDataModule.OtherWayTable.Active      := False;
    RoutesDataModule.OtherWayTable.TableName   := OtherWayEdit.Text+'.db';
    RoutesDataModule.OtherWayTable.Active      := True;
  end else
    RoutesDataModule.RoutesTable.Active      := False;
end;

procedure TNewRouteWizard.FormResize(Sender: TObject);
begin
  Notebook.Width := Width;
end;

procedure TNewRouteWizard.RouteBackPageProxyLeave(Next: Boolean);
begin
  RoutesDataModule.OtherWayTable.Active      := False;
end;



procedure TNewRouteWizard.RouteNameEditKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
const ValidChars = [61..90, VK_BACK, VK_CANCEL,VK_LEFT,VK_RIGHT,VK_HOME, VK_END];
begin
  if not (Key in ValidChars) then begin
    MessageBeep(1);
    Key := 0;
  end;
end;


procedure TNewRouteWizard.RouteNameEditKeyPress(Sender: TObject;
  var Key: Char);
var ValidChars : set of char;

begin
  ValidChars := ['a'..'z','A'..'Z',Chr(VK_BACK),Chr(VK_CANCEL)];
  if not (Key in ValidChars) then begin
    MessageBeep(1);
    Key := #0;
  end;
end;

procedure TNewRouteWizard.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Notebook.PageIndex := 0;
  RouteNameEdit.Text := '';
  OtherWayEdit.Text  := '';
end;

end.
