{*****************************************************************************
 *
 *  uSearchRoutine.pas - Routines to perform search in graph
 *
 *  Copyright (c) 1999 Diego Amicabile
 *
 *  Author:     Diego Amicabile
 *  E-mail:     diegoami@yahoo.it
 *  Homepage:   http://www.geocities.com/diegoami
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation;
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *----------------------------------------------------------------------------
 *
 *  Revision history:
 *
 *     DATE     REV                 DESCRIPTION
 *  ----------- --- ----------------------------------------------------------
 *
 *****************************************************************************}

unit uSearchRoutine;

interface

uses ObjGraph;

type
TTownNode = class(TGraphNode)
  Town : TStringObj;
  Hour : String;
  Day : Char;
  constructor Create(Town : TStringObj);
  procedure SumDistance(distance : real);
  procedure Update(FormerNode : TGraphNode; Distance : real); override;
end;

TRouteEdge = class(TGraphEdge)
  Route : String;
  Vehicle : String;
  DepHour : String;

  constructor Create(FromNode, ToNode : TGraphNode; InRoute : String);

  procedure SetWeightFrom(FromNode : TGraphNode); override;
end;

function HourDiff(HourStart, HourEnd : String) : integer;
function HourToString(Hour : String) : String;
function DayToChar(Day : String) : Char;
function validHour(Hour : String) : boolean;

implementation

uses DataMod, sysutils;


function validHour(Hour : String) : boolean;
var hh,mm : Integer;
  RS : String;
begin
  RS := HourToString(Hour);
  hh := StrToInt(Copy(RS,1,2));
  mm := StrToInt(Copy(RS,4,2));
  if (hh > 23) or (hh < 0) or (mm > 59) or (mm < 0) then
    result := false
  else
    result := true;
end;


function DayToChar(Day : String) : Char;
begin
  Day := LowerCase(Day);
  if Day = 'sunday' then result        := 'u'
  else if Day = 'thursday' then result := 'h'
  else if Day = '' then result         := ' '
  else result := Day[1];
end;

function NextDay(ch : Char) : Char;
var chr : char;
begin
  case Ch of
    'm' : chr := 't';
    't' : chr := 'w';
    'w' : chr := 'h';
    'h' : chr := 'f';
    'f' : chr := 's';
    's' : chr := 'u';
    'u' : chr := 'm';
    ' ' : chr := ' ';
  end;
  result := chr;
end;

constructor TRouteEdge.Create(FromNode, ToNode : TGraphNode; InRoute : String);
begin
  inherited Create(FromNode, ToNode);
  Route := InRoute;
end;

procedure TRouteEdge.SetWeightFrom(FromNode : TGraphNode);
var Hour, HourTo :  String;
    Town, TownTo : String;
begin
  Hour := (FromNode As TTownNode).Hour;
  HourTo := RoutesDataModule.GetHourTo(Self, Hour);
  Weight := HourDiff(Hour, HourTo);
end;

procedure TTownNode.SumDistance(distance : real);
var ho, mo : real;
  hs, ms ,he, me : String;
begin
  hs := Copy(Hour,1,2);
  ms := Copy(Hour,4,2);
  mo := StrToFloat(ms) + distance;
  ho := StrToFloat(hs) + Round(mo) div 60;
  mo := Round(mo) mod 60;
  if ho > 24 then begin
    ho := ho - 24;
    Day := NextDay(Day);
  end;
  hs := FloatToStr(Ho);
  ms := FloatToStr(Mo);
  if Length(hs) = 1 then
    hs := '0'+hs;
  if Length(ms) = 1 then
    ms := '0'+ms;
  Hour := hs+'.'+ms;
end;

procedure TTownNode.Update(FormerNode : TGraphNode; Distance : real);
begin
  Hour := (FormerNode As TTownNode).Hour;
  Day := (FormerNode As TTownNode).Day;
  SumDistance(Distance);
end;

constructor TTownNode.Create(Town : TStringObj);
begin
  inherited Create(nil);
  Self.Town := Town;
end;



function HourDiff(HourStart, HourEnd : String) : integer;
var hh, mm : integer;
     hs, ms ,he, me : String;
     hsi, hei, msi, mei : integer;
begin
  hs := Copy(HourStart,1,2);
  he := Copy(HourEnd,1,2);
  ms := Copy(HourStart,4,2);
  me := Copy(HourEnd,4,2);
  hsi := StrToInt(hs);
  hei := StrToInt(he);
  msi := StrToInt(ms);
  mei := StrToInt(me);
  if hsi > hei then
    hei := hei+24;
  hh := hei-hsi;
  mm := mei-msi;
  result := hh*60+mm;
end;

function HourToString(Hour : String) : String;
var sh : string;
begin
  if Pos('.',Hour) = 3 then
    sh := Copy(Hour,1,2)
  else
    sh := '0'+Copy(Hour,1,1);
  if (Pos('PM',Hour) > 0) and (sh <> '12') then
    sh := IntToStr(StrToInt(sh)+12);
  if Pos('.',Hour) = 3 then
    result := sh + '.'+Copy(Hour,4,2)
  else
    result := sh + '.'+Copy(Hour,3,2);
end;


end.
