{*****************************************************************************
 *
 *  Search.pas - Dialog to perform Search
 *
 *  Copyright (c) 1999 Diego Amicabile
 *
 *  Author:     Diego Amicabile
 *  E-mail:     diegoami@yahoo.it
 *  Homepage:   http://www.geocities.com/diegoami
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation;
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *----------------------------------------------------------------------------
 *
 *  Revision history:
 *
 *     DATE     REV                 DESCRIPTION
 *  ----------- --- ----------------------------------------------------------
 *
 *****************************************************************************}

unit uSearchUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Db, DBTables, HTTPApp, shellapi, Mask, ComCtrls;
{search result disabled - to enable it set ResultMemo to Visible }
type
  TConnection = record
    TownFrom, TownTo  : String;
    Name  : String
  end;
  PConnection = ^TConnection;



  ConnectionList  : TList;
  function GetTownsInRoute(Town, Route : String) TStringList;
  function GetRoutesFromTown(Town : String) : TStringList;
  //function GetFollowingVehicles(Route, TownIn, TownOut,
  //       Hour : String; var Vehicle : String) : String;
  function HourToString(Hour : String) : String;
    function ValidDay(DayChar : String) : boolean;

  public
    { Public declarations }
  end;
var
  SearchForm: TSearchForm;


implementation

uses DataMod;

{$R *.DFM}

function DayToChar(Day : String) : Char;
begin
  Day := LowerCase(Day);
  if Day = 'sunday' then result        := 'u'
  else if Day = 'thursday' then result := 'h'
  else if Day = '' then result         := ' '
  else result := Day[1];
end;



function TSearchForm.HourToString(Hour : String) : String;
var sh : string;
begin
  if Pos(':',Hour) = 3 then
    sh := Copy(Hour,1,2)
  else
    sh := '0'+Copy(Hour,1,1);
  if (Pos('PM',Hour) > 0) and (sh <> '12') then
    sh := IntToStr(StrToInt(sh)+12);
  if Pos(':',Hour) = 3 then
    result := sh + ':'+Copy(Hour,4,2)
  else
    result := sh + ':'+Copy(Hour,3,2);
end;

function TSearchForm.ValidDay(DayChar : String) : boolean;
var DaySearch : String;
    CharSearched : Char;
begin
  DaySearch := Combobox.Items[ComboBox.ItemIndex];
  CharSearched := DayToChar(DaySearch);
  if DayChar = '' then
    result := true
  else if Pos(CharSearched,DayChar) > 0 then
    result := true
  else
    result := false;
end;

function TSearchForm.GetFollowingVehicles(Route, TownIn, TownOut,
         Hour : String; var Vehicle : String) : String;
var      FHOur, PHour, DayChar : String;

begin
  RoutesDataModule.HourQuery.Close;
  RoutesDataModule.HourQuery.SQL.Clear;
  RoutesDataModule.HourQuery.SQL.Add('SELECT CAST('+TownOut+ ' AS CHAR(11)),CAST('+TownIn+ ' AS CHAR(11)), DAYS, VEHICLE FROM ' + Route);
  RoutesDataModule.HourQuery.Open;
  RoutesDataModule.HourQuery.First;
  FHour := HourToString(RoutesDataModule.HourQuery.Fields[1].AsString);
  PHour := HourToString(RoutesDataModule.HourQuery.Fields[0].AsString);
  DayChar := RoutesDataModule.HourQuery.Fields[2].AsString;
  Vehicle := RoutesDataModule.HourQuery.Fields[3].AsString;
  if (Phour < FHour) then begin
    result := '00:00';
    exit;
  end;
  while (not RoutesDataModule.HourQuery.EOF)
        and ((FHour < Hour) or not(ValidDay(DayChar))) do begin
    FHour := HourToString(RoutesDataModule.HourQuery.Fields[1].AsString);
    PHour := HourToString(RoutesDataModule.HourQuery.Fields[0].AsString);
    DayChar := RoutesDataModule.HourQuery.Fields[2].AsString;
    Vehicle := RoutesDataModule.HourQuery.Fields[3].AsString;
    RoutesDataModule.HourQuery.Next;
  end;
  if (FHour >= Hour) and (ValidDay(DayChar)) then
    if TownIn = TownOut then
      result := FHour
    else
      result := Phour
  else
    result := '00:00';
end;

procedure TSearchForm.SearchButtonClick(Sender: TObject);
var Departure, Arrival, Hour : String;
    Routes                   : TStringList;
    found, everfound             : boolean;
    level              : smallint;

  function PlusMinute(Hour : String) : String;
  var H,M : smallint;
      Hs,Ms : String;
  begin
    H := StrToInt(Copy(Hour,1,2));
    M := StrToInt(Copy(Hour,4,2));
    M := M + 1;
    if M = 60 then begin
      M := 0;
      H := H+1;
    end;
    Hs := IntToStr(H);
    if Length(Hs) = 1 then
      Hs := '0'+Hs;

    Ms := IntToStr(M);
    if Length(Ms) = 1 then
      Ms := '0'+Ms;

    result := Hs+':'+Ms;
  end;

  function GetConnection(Town : String; Route : String;
           level : smallint; Hour : String) : boolean;
  var Towns                   : TStringList;
      Routes,Hours            : TStringList;
      i,j                     : integer;
      found                   : boolean;
      Connection            : PConnection;
      Vehicle               : String;
    procedure WriteSolution(Nodes : TStringList);
    var i : integer;
    begin
{      for i := 0 to Nodes.Count-1 do
        ResultMemo.Lines.Add(Nodes.Strings[i]);
      ResultMemo.Lines.Add('');}
    end;

  begin
    found                       := false;
    result                      := false;
    Towns := GetTownsInRoute(Town,Route,Hour,Hours, Vehicle);
    if level = 1 then begin
      if Towns.IndexOf(Arrival) >= 0 then begin
        New(Connection);
        result := true;
        Connection^.TownFrom := Town;
        Connection^.TownTo   := Arrival;
        Connection^.HourFrom := Hours.Strings[Towns.IndexOf(Town)];
        Connection^.HourTo   := Hours.Strings[Towns.IndexOf(Arrival)];
        Connection^.Name     := Vehicle;
        ConnectionList.add(Connection);
       end else
         result := false;

    end else begin
      if Towns.IndexOf(Arrival) >= 0 then begin
        New(Connection);
        Connection^.TownFrom := Town;
        Connection^.TownTo   := Arrival;
        Connection^.HourFrom := Hours.Strings[Towns.IndexOf(Town)];
        Connection^.HourTo   := Hours.Strings[Towns.IndexOf(Arrival)];
        Connection^.Name     := Vehicle;
        ConnectionList.add(Connection);
        result := true;
      end else
        for i := 0 to Towns.Count-1 do begin
          if Towns.Strings[i] <> Town then begin
            Routes    := GetRoutesFromTown(Towns.Strings[i]);
            for j := 0 to Routes.Count-1 do begin
              if Routes.Strings[j] <> Route then begin
                found := GetConnection(Towns.Strings[i], Routes.Strings[j],level-1, Hours.Strings[i]);
                if found then begin
                   result := true;
                   New(Connection);
                   Connection^.TownFrom := Town;
                   Connection^.TownTo   := Towns.Strings[i];
                   Connection^.HourFrom := Hours.Strings[Towns.IndexOf(Town)];
                   Connection^.HourTo   := Hours.Strings[Towns.IndexOf(Towns.Strings[i])];
                   Connection^.Name     := Vehicle;
                   ConnectionList.add(Connection);
                   found := false;
                end;
              end;
            end;
          end;
        end;
    end;
    Towns.Destroy;
    Hours.Destroy;
  end;

var
  I,J : integer;
  ConnString : String;
  Connection : TConnection;
  DepartureHour : String;
  Noconnection   : boolean;
  ListItem       : TListItem;
  ListLines      : TStringList;
begin
  ProcessingLabel.Visible := true;
  ProcessingLabel.Caption := 'Processing ...';

  ListLines := TStringList.Create;
  ListView.Items.Clear;
  Found := false;
  ConnectionList := TList.Create;
  Hour      := HourEdit.Text;
  Departure := DepartureComboBox.Items[DepartureComboBox.ItemIndex];
  Arrival   := ArrivalComboBox.Items[ArrivalComboBox.ItemIndex];
  Routes    := GetRoutesFromTown(Departure);

    Level := 1;
    Everfound := false;
    repeat
      for i := 0 to Routes.Count-1 do begin
        Found := GetConnection(Departure, Routes.Strings[i],Level,Hour);
        if (Found) then begin
          Everfound := true;
          DepartureHour := PConnection(ConnectionList.Items[ConnectionList.Count-1])^.HourFrom;
          ListView.Items.BeginUpdate;
          for j := ConnectionList.Count-1 downto 0 do begin
            Connection := PConnection(ConnectionList.Items[j])^;

            Connstring := 'Vehicle ' + Connection.Name + ' leaves from ' +Connection.TownFrom+' at '+Connection.HourFrom;
            ConnString := ConnString + ' and reaches ' + Connection.TownTo + ' at ' + Connection.HourTo +' on ' + ComboBox.Items[ComboBox.ItemIndex] ;
            if ListLines.indexOf(ConnString) = -1 then begin
              ListLines.Add(ConnString);
              ListItem := ListView.Items.Add;
              ListItem.Caption := Connection.Name;
              ListItem.SubItems.Add(Connection.TownFrom);
              ListItem.SubItems.Add(Connection.HourFrom);
              ListItem.SubItems.Add(Connection.TownTo);
              ListItem.SubItems.Add(Connection.HourTo);
            end;
          end;

          ListView.Items.EndUpdate;
          ConnectionList.Clear;
        end;
      end;
      inc(level);
    until (Everfound) or (Level >= 5);
    if Everfound then
      Hour := PlusMinute(DepartureHour);
  Routes.Destroy;
  ConnectionList.Destroy;
  if BrowserCheckBox.Checked then
    SubmitButtonClick(Sender);
  ProcessingLabel.Visible := false;

end;




procedure TSearchForm.SubmitButtonClick(Sender: TObject);
var ToPass : String;
begin
  ToPass := URLEdit.Text+'&Departure='+DepartureComboBox.Items[DepartureComboBox.ItemIndex];
  ToPass := ToPass+'&Arrival='+ArrivalComboBox.Items[ArrivalComboBox.ItemIndex];
  ToPass := ToPass+'&Hour='+Copy(HourEdit.Text,1,2);
  ToPass := ToPass+'&Minute='+Copy(HourEdit.Text,4,2);
  ToPass := ToPass+'&WeekDay='+ComboBox.Items[ComboBox.ItemIndex];
  Shellexecute(0,nil,Pchar(ToPass),nil,nil,SW_NORMAL);
end;

procedure TSearchForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  HourEdit.Text := '';
  RoutesDataModule.URLLocation := URLEdit.Text;
end;

procedure TSearchForm.FormShow(Sender: TObject);
begin
  
  URLEdit.Text := RoutesDataModule.URLLocation;
  with RoutesDataModule do begin
    TownListQuery.Close;
    TownListQuery.Open;
    TownListQuery.First;
    DepartureComboBox.Items.Clear;
    ArrivalComboBox.Items.Clear;
    while not TownListQuery.Eof do begin
      DepartureComboBox.Items.Add(TownListQuery.Fields[0].AsString);
      ArrivalComboBox.Items.Add(TownListQuery.Fields[0].AsString);

      TownListQuery.Next;
    end;
  end;

end;

procedure TSearchForm.ListViewCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
var Time1, Time2 : String;
begin
  Time1 := Item1.SubItems.Strings[1];
  Time2 := Item2.SubItems.Strings[1];
  Compare := CompareStr(Time1,Time2);
end;

end.
